@@ -582,7 +582,7 @@ gnetwork_tcp_server_set_property (GObject * object, guint property, const GValue
 
 	if (interface != NULL)
 	  {
-	    server->_priv->interface = g_strdup (server->_priv->interface);
+	    server->_priv->interface = g_strdup (interface);
 	    info = gnetwork_interface_get_info (interface);
 	  }
 	else
