@@ -379,10 +379,18 @@ io_channel_handler (GIOChannel * channel, GIOCondition cond, GNetworkUdpDatagram
       /* The out buffer is empty, stop watching G_IO_OUT */
       if (udp->_priv->buffer == NULL)
 	{
-	  gnetwork_thread_source_remove (udp->_priv->source_id);
-	  udp->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_HUP | G_IO_ERR);
-	  udp->_priv->source_id = gnetwork_thread_io_add_watch (channel, udp->_priv->source_cond,
-								(GIOFunc) io_channel_handler, udp);
+	  if (udp->_priv->source_id != 0)
+	     gnetwork_thread_source_remove (udp->_priv->source_id);
+
+	  if (udp->_priv->channel != NULL)
+	    {
+	      udp->_priv->source_cond = (G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP);
+	      udp->_priv->source_id =
+		gnetwork_thread_io_add_watch (udp->_priv->channel,
+					      udp->_priv->source_cond,
+					      (GIOFunc) io_channel_handler, udp);
+	    }
+
 	  retval = FALSE;
 	}
     }
@@ -1084,7 +1092,7 @@ gnetwork_udp_datagram_dispose (GObject * object)
   GNetworkUdpDatagram *udp = GNETWORK_UDP_DATAGRAM (object);
 
   if (udp->_priv->dgram_status > GNETWORK_DATAGRAM_CLOSED)
-    gnetwork_datagram_close (udp);
+    gnetwork_datagram_close (GNETWORK_DATAGRAM (udp));
 
   gnetwork_interface_info_unref (udp->_priv->interface_info);
 
